import requests
from urllib.parse import urljoin

class RestAPI(requests.Session):
    """
    Encapsulates all REST calls to the Gardena API.
    """
    
    # The base URL for all Gardena requests. Change this if you want to fire the requests against other services
    base_url = "https://sg-api.dss.husqvarnagroup.net/sg-1/"
    
    def __init__(self, *args, **kw):
        super().__init__(*args, **kw)
        self.headers["Accept"] = "application/json"
    
    def request(self, method, url, *args, **kw):
        return super().request(method, urljoin(self.base_url, url), *args, **kw)
    
    def post_sessions(self, email_address, password):
        """
        Posts session data to the server.
        """
        response = self.post('sessions', json={
            'sessions': {
                'email': email_address,
                'password': password,
            }
        })
        return response.json()
    
    def get_locations(self, user_id):
        """
        Gets all the locations that are associated with a given user ID.
        """
        response = self.get("locations", params={
            'user_id': user_id,
        })
        return response.json()
    
    def get_devices(self, location_id):
        """
        Loads the devices from a given location ID.
        """
        response = self.get('devices', params={
            'locationId': location_id
        })
        return response.json()
    
    def post_command(self, device, command_name, parameters=None):
        data = {'name': command_name}
        if parameters is not None:
            data['parameters'] = parameters

        url = 'devices/{device.id}/abilities/{device.category}/command'.format(device=device)
        return self.post(url, params={
            'locationId': device.location.id
        }, json=data)


from wt.pygardena.rest_api import *

import unittest
import requests
import requests_mock
    
class MockLocation:
    id = 'washington'

class MockDevice:
    location = MockLocation()
    id = 'black-box'
    category = 'code-breaker'

@requests_mock.mock()
class TestRestAPI(unittest.TestCase):

    verified = {'result': 'verified'}
    
    def setUp(self):
        self.under_test = RestAPI()

    def test_relative_url(self, mock):
        mock.get('https://smart.gardena.com/sg-1/test', json=self.verified)
        self.assertEqual(self.under_test.get('test').json(), self.verified)
    
    def test_accepts_json(self, mock):
        mock.get('https://smart.gardena.com/sg-1/test', request_headers={'Accept': 'application/json'}, json=self.verified)
        self.assertEqual(self.under_test.get('test').json(), self.verified)
    
    def test_post_sessions(self, mock):
        def match_sessions(request):
            return request.json() == { 'sessions': {
                'email': 'werner.brandes@playtronics.example',
                'password': 'MyVoiceIsMyPassport',
            }}
        mock.post('https://smart.gardena.com/sg-1/sessions', additional_matcher=match_sessions, json=self.verified)
        self.assertEqual(self.under_test.post_sessions('werner.brandes@playtronics.example', 'MyVoiceIsMyPassport'), self.verified)
    
    def test_get_locations(self, mock):
        mock.get('https://smart.gardena.com/sg-1/locations?user_id=martin-bishop', json=self.verified)
        self.assertEqual(self.under_test.get_locations('martin-bishop'), self.verified)
    
    def test_get_devices(self, mock):
        mock.get('https://smart.gardena.com/sg-1/devices?locationId=washington', json=self.verified)
        self.assertEqual(self.under_test.get_devices('washington'), self.verified)
    
    def test_post_command_without_parameters(self, mock):
        def match_body(request):
            return request.json() == {
                'name': 'open'
            }
        mock.post('https://smart.gardena.com/sg-1/devices/black-box/abilities/code-breaker/command?locationId=washington', additional_matcher=match_body, json=self.verified)
        self.assertEqual(self.under_test.post_command(MockDevice(), 'open').json(), self.verified)
    
    def test_post_command_with_parameters(self, mock):
        def match_body(request):
            return request.json() == {
                'name': 'open',
                'parameters': {'force': True}
            }
        mock.post('https://smart.gardena.com/sg-1/devices/black-box/abilities/code-breaker/command?locationId=washington', additional_matcher=match_body, json=self.verified)
        self.assertEqual(self.under_test.post_command(MockDevice(), 'open', {'force': True}).json(), self.verified)
    


from .device import *

class GardenaSmartWateringComputer(GardenaSmartDevice):
    def __init__(self, rest_api, location, raw_data):
        super().__init__(rest_api, location, raw_data)
        self.category = "outlet"
        self._valve_open = self.get_value_of_property('watering_outlet', 'valve_open')

    def update(self):
        super().update()
        self._valve_open = self.get_value_of_property('watering_outlet', 'valve_open')

    def get_battery_status(self):
        return self.get_value_of_property('battery_power', 'disposable_battery_status')
    def get_ambient_temperature(self):
        return self.get_value_of_property('ambient_temperature_sensor', 'temperature')
    def get_ambient_frost_warning(self):
        return self.get_value_of_property('ambient_temperature_sensor', 'frost_warning')
    def get_valve_open(self):
        return self._valve_open
    def get_manual_override(self):
        return self.get_value_of_property('watering_outlet', 'manual_override')
    def get_button_manual_override_time(self):
        return self.get_value_of_property('watering_outlet', 'button_manual_override_time')
    def get_last_manual_override_time(self):
        return self.get_value_of_property('watering_outlet', 'last_manual_override_time')
    def get_scheduled_watering_next_start(self):
        return self.get_value_of_property('scheduling', 'scheduled_watering_next_start')
    def get_scheduled_watering_end(self):
        return self.get_value_of_property('scheduling', 'scheduled_watering_end')
    def get_adaptive_scheduling_last_decision(self):
        return self.get_value_of_property('scheduling', 'adaptive_scheduling_last_decision')

    def get_generic_info(self):
        device_info = super().get_info()
        #add sensor specific details to device info
        device_info['battery_status'] = self.get_battery_status()
        return device_info

    def get_info(self):
        device_info = self.get_generic_info()
        device_info['ambient_temperature'] = self.get_ambient_temperature()
        device_info['ambient_frost_warning'] = self.get_ambient_frost_warning()
        device_info['valve_open'] = self.get_valve_open()
        device_info['manual_override'] = self.get_manual_override()
        device_info['button_manual_override_time'] = self.get_button_manual_override_time()
        device_info['last_manual_override_time'] = self.get_last_manual_override_time()
        device_info['scheduled_watering_next_start'] = self.get_scheduled_watering_next_start()
        device_info['scheduled_watering_end'] = self.get_scheduled_watering_end()
        device_info['adaptive_scheduling_last_decision'] = self.get_adaptive_scheduling_last_decision()
        return  device_info

    def start(self, duration=30):
        self.send_command('manual_override', {'duration': duration})
        self._valve_open = True

    def stop(self):
        self.send_command('cancel_override')
        self._valve_open = False




from .account import *
from .mower import *
from .sensor import *
from .watering_computer import *
import json

class GardenaSmartLocation:
    def __init__(self, rest_api, raw_data):
        self.rest_api = rest_api
        self.raw_data = raw_data
        self.raw_devices = None
        self.name = raw_data['name']
        self.id = raw_data['id']
        self.devices_mower = set()
        self.devices_sensor = set()
        self.devices_watering_computer = set()
        self.load_devices()

    def get_id(self):
        return self.id

    def get_name(self):
        return self.name

    def get_latitude(self):
        return self.raw_data['geo_position']['latitude']

    def get_longitude(self):
        return self.raw_data['geo_position']['longitude']

    def get_address(self):
        return self.raw_data['geo_position']['address']

    def get_city(self):
        return self.raw_data['geo_position']['city']

    def get_sunrise(self):
        return self.raw_data['geo_position']['sunrise']

    def get_sunset(self):
        return self.raw_data['geo_position']['sunset']

    def get_info(self):
        location_info = {}
        location_info['id'] = self.get_id()
        location_info['name'] = self.get_name()
        location_info['latitude'] = self.get_latitude()
        location_info['longitude'] = self.get_longitude()
        location_info['address'] = self.get_address()
        location_info['city'] = self.get_city()
        location_info['sunrise'] = self.get_sunrise()
        location_info['sunset'] = self.get_sunset()
        return location_info

    def update_devices(self):
        self.update_raw_data()
        for device in self.devices_mower:
            device.update()
        for device in self.devices_sensor:
            device.update()
        for device in self.devices_watering_computer:
            device.update()

    def update_raw_data(self):
        self.raw_devices = self.rest_api.get_devices(self.id)

    def get_raw_device_data(self, device_id):
        for device in self.raw_devices['devices']:
            if device['id'] == device_id:
                return device
        _LOGGER.warn('raw data not found for device: '+device_id)

    def get_mowers(self):
        return self.devices_mower

    def get_sensors(self):
        return self.devices_sensor

    def get_watering_computers(self):
        return self.devices_watering_computer

    def load_devices(self):
        self.update_raw_data()
        for device in self.raw_devices['devices']:
            if device['category'] == 'mower':
                self.devices_mower.add(GardenaSmartMower(self.rest_api, self, device))
            if device['category'] == 'sensor':
                self.devices_sensor.add(GardenaSmartSensor(self.rest_api, self, device))
            if device['category'] == 'watering_computer':
                self.devices_watering_computer.add(GardenaSmartWateringComputer(self.rest_api, self, device))





from .account import *

class GardenaSmartDevice:
    def __init__(self, rest_api, location, raw_data):
        self.rest_api = rest_api
        self.location = location
        self.raw_data = raw_data
        self.name = raw_data['name']
        self.zone = None
        self.id = raw_data['id']
        self.category = 'unknown'
        self.update()

    def get_value_of_property(self, ability, property):
        for data_ability in self.raw_data['abilities']:
            if data_ability['type'] != ability:
                continue
            for data_property in data_ability['properties']:
                if data_property['name'] == property:
                    return data_property['value']
        return None

    def update(self):
        self.raw_data = self.location.get_raw_device_data(self.id)

    def get_category(self):
        return self.category

    def get_battery_level(self):
        return self.get_value_of_property('battery_power', 'level')

    def get_radio_quality(self):
        return self.get_value_of_property('radio_link', 'quality')

    def get_radio_connection_status(self):
        return self.get_value_of_property('radio_link', 'connection_status')

    def get_radio_state(self):
        return self.get_value_of_property('radio_link', 'state')

    def get_info(self):
        device_info = {}
        device_info['category'] = self.get_category()
        device_info['battery_level'] = self.get_battery_level()
        device_info['radio_quality'] = self.get_radio_quality()
        device_info['radio_connection_status'] = self.get_radio_connection_status()
        device_info['radio_state'] = self.get_radio_state()
        return device_info

    def send_command(self, name, parameters=None):
        self.rest_api.post_command(self, name, parameters)


from .device import *

class GardenaSmartSensor(GardenaSmartDevice):
    def __init__(self, rest_api, location, raw_data):
        super().__init__(rest_api, location, raw_data)
        self.category = "sensor"


    def get_battery_status(self):
        return self.get_value_of_property('battery_power', 'disposable_battery_status')
    def get_ambient_temperature(self):
        return self.get_value_of_property('ambient_temperature_sensor', 'temperature')
    def get_ambient_frost_warning(self):
        return self.get_value_of_property('ambient_temperature_sensor', 'frost_warning')
    def get_soil_temperature(self):
        return self.get_value_of_property('soil_temperature_sensor', 'temperature')
    def get_soil_humidity(self):
        return self.get_value_of_property('soil_humidity_sensor', 'humidity')
    def get_light(self):
        return self.get_value_of_property('light_sensor', 'light')

    def get_generic_info(self):
        device_info = super().get_info()
        #add sensor specific details to device info
        device_info['battery_status'] = self.get_battery_status()
        return device_info

    def get_info(self):
        device_info = self.get_generic_info()
        device_info['ambient_temperature'] = self.get_ambient_temperature()
        device_info['ambient_frost_warning'] = self.get_ambient_frost_warning()
        device_info['soil_temperature'] = self.get_soil_temperature()
        device_info['soil_humidity'] = self.get_soil_humidity()
        device_info['light'] = self.get_light()
        return  device_info



from .device import *

class GardenaSmartMower(GardenaSmartDevice):
    def __init__(self, rest_api, location, raw_data):
        super().__init__(rest_api, location, raw_data)
        self.category = "mower"

    def start(self, duration=1440):
        self.send_command('start_override_timer', {'duration': duration})

    def park_until_timer(self):
        self.send_command('park_until_next_timer')

    def park(self):
        self.send_command('park_until_further_notice')

    def get_manual_operation(self):
        return self.get_value_of_property('robotic_mower', 'manual_operation')
    def get_status(self):
        return self.get_value_of_property('robotic_mower', 'status')
    def get_error(self):
        return self.get_value_of_property('robotic_mower', 'error')
    def get_battery_charging(self):
        return self.get_value_of_property('battery_power', 'charging')
    def get_last_error_code(self):
        return self.get_value_of_property('robotic_mower', 'last_error_code')
    def get_source_for_next_start(self):
        return self.get_value_of_property('robotic_mower', 'source_for_next_start')
    def get_timestamp_next_start(self):
        return self.get_value_of_property('robotic_mower', 'timestamp_next_start')
    def get_cutting_time(self):
        return self.get_value_of_property('robotic_mower_stats', 'cutting_time')
    def get_charging_cycles(self):
        return self.get_value_of_property('robotic_mower_stats', 'charging_cycles')
    def get_collisions(self):
        return self.get_value_of_property('robotic_mower_stats', 'collisions')
    def get_running_time(self):
        return self.get_value_of_property('robotic_mower_stats', 'running_time')

    def get_info(self):
        device_info = super().get_info()
        #add mower specific details to device info
        device_info['manual_operation'] = self.get_manual_operation()
        device_info['status'] = self.get_status()
        device_info['error'] = self.get_error()
        device_info['battery_charging'] = self.get_battery_charging()
        device_info['last_error_code'] = self.get_last_error_code()
        device_info['source_for_next_start'] = self.get_source_for_next_start()
        device_info['timestamp_next_start'] = self.get_timestamp_next_start()
        device_info['cutting_time'] = self.get_cutting_time()
        device_info['charging_cycles'] = self.get_charging_cycles()
        device_info['collisions'] = self.get_collisions()
        device_info['running_time'] = self.get_running_time()
        return  device_info



import setuptools
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='wt.pygardena',
    version='0.9.6',
    packages=setuptools.find_packages(),
    url='https://github.com/wijnandtop/wt.pygardena',
    license='Apache License 2.0',
    author='Wijnand Top',
    author_email='wijnand@bammes.nl',
    description='Library to connect to Garena Smart',
    long_description=long_description,
    long_description_content_type='text/markdown',
    python_requires=">=3.4",
    install_requires=[
        'requests',
    ],
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: Apache Software License",
        "Natural Language :: English",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Topic :: Home Automation",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    tests_require = ['requests-mock']
)



from .location import *
from .rest_api import RestAPI

class GardenaSmartAccount:
    
    def __init__(self, email_address, password):
        self.email_address = email_address
        self.password = password
        
        self.rest_api = RestAPI()
        
        self.locations = set()
        self.raw_locations = None
        self.update_authtokens()


    def load_locations(self):
        self.raw_locations = response_data = self.rest_api.get_locations(self.userID)
        for location in response_data['locations']:
            self.locations.add(GardenaSmartLocation(self.rest_api, location))

    def get_locations(self):
        if len(self.locations) == 0:
            self.load_locations()
        return self.locations

    def get_raw_location_data(self, location_id):
        for location in self.raw_locations:
            if location.id == location_id:
                return location
        raise KeyError(location_id)

    def update_devices(self):
        for location in self.locations:
            location.update_devices()
    
    def update_authtokens(self):
        """
        Get authentication token from servers and store it as the session in the Rest API.
        """
        response_data = self.rest_api.post_sessions(self.email_address, self.password)
        self.AuthToken = response_data['sessions']['token']
        self.refreshToken = response_data['sessions']['refresh_token']
        self.userID = response_data['sessions']['user_id']
        self.rest_api.headers['X-Session'] = self.AuthToken

    def get_all_mowers(self):
        all_mowers = set()
        for location in self.get_locations():
            for mower in location.get_mowers():
                all_mowers.add(mower)
        return all_mowers

    def get_all_sensors(self):
        all_sensors = set()
        for location in self.get_locations():
            for sensor in location.get_sensors():
                all_sensors.add(sensor)
        return all_sensors

    def get_all_watering_computers(self):
        all_sensors = set()
        for location in self.get_locations():
            for watering_computer in location.get_watering_computers():
                all_sensors.add(watering_computer)
        return all_sensors


